using System;
using System.ComponentModel;
using System.Drawing;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Runtime.InteropServices;
using AjaraInit;

namespace CAjrCustCtrl
{
	//----------------------------------------------------------------------------------------------------------//
	[ToolboxBitmap(typeof(CAjrSockServer), "CAjrSockServer.bmp")]
	public partial class CAjrSockServer : Component
	{
		//----- Oւ̃Cxg -------------------------------------------------------------------------------//
		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notify the received text data.")]
		public event SsvEvtRxText			OnRxText;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notifies the received escape sequence.")]
		public event SsvEvtRxEsc			OnRxEsc;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notify the received control code.")]
		public event SsvEvtRxCtrl			OnRxCtrl;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notify the received packet data.")]
		public event SsvEvtRxPacket			OnRxPacket;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notifies the part of the received data other than the packet.")]
		public event SsvEvtRxNoPkt			OnRxNoPkt;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notify the completion of transmission.")]
		public event SsvEvtTxEmpty			OnTxEmpty;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notify incoming chunk text in real time.")]
		public event SsvEvtRxChunkTxt		OnRxChunkTxt;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notify incoming chunk binary data in real time.")]
		public event SsvEvtRxChunkBin		OnRxChunkBin;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notifies that invalid text chunk data has been received.")]
		public event SsvEvtRxInvChunk		OnRxInvChunk;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notifies you that you have connected to a client.")]
		public event SsvEvtConnect			OnConnect;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notifies connection with a client has been disconnected.")]
		public event SsvEvtDisconnect		OnDisconnect;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notifies that the server has started.")]
		public event SsvEvtStart			OnStart;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notifies that the server has stopped.")]
		public event SsvEvtStop				OnStop;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notifies that a reception error has been detected.")]
		public event SsvEvtRecvError		OnRecvError;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notifies that a transmission error has been detected.")]
		public event SsvEvtSendError		OnSendError;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Notifies that a general error has been detected.")]
		public event SsvEvtGeneralError		OnGeneralError;

		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Clients enumeration notification.")]
		public event SsvEvtEnumClients		OnEnumClients;

		private SsvCbkRxText				m_SsvCbkRxText;
		private SsvCbkRxEsc					m_SsvCbkRxEsc;
		private SsvCbkRxCtrl				m_SsvCbkRxCtrl;
		private SsvCbkRxPacket				m_SsvCbkRxPacket;
		private SsvCbkRxNoPkt				m_SsvCbkRxNoPkt;
		private SsvCbkTxEmpty				m_SsvCbkTxEmpty;
		private SsvCbkRxChunkTxt			m_SsvCbkRxChunkTxt;
		private SsvCbkRxChunkBin			m_SsvCbkRxChunkBin;
		private SsvCbkRxInvChunk			m_SsvCbkRxInvChunk;
		private SsvCbkConnect				m_SsvCbkConnect;
		private SsvCbkDisconnect			m_SsvCbkDisconnect;
		private SsvCbkStart					m_SsvCbkStart;
		private SsvCbkStop					m_SsvCbkStop;
		private SsvCbkRecvError				m_SsvCbkRecvError;
		private SsvCbkSendError				m_SsvCbkSendError;
		private SsvCbkGeneralError			m_SsvCbkGeneralError;
		private SsvCbkEnumClients			m_SsvCbkEnumClients;

		//----- s[h ---------------------------------------------------------------------------------//
		private ESsvMode		m_SsvMode	= ESsvMode.NotificationByEvent;	//	s[h
		private ESsvChunkMode	m_ChunkMode	= ESsvChunkMode.BinaryData;		//	`N

		//----- NCAgf[^ MAP	------------------------------------------------------------------------//
		private IDictionary<IntPtr, object> m_mapClientData = new Dictionary<IntPtr, object>();

		//----- [N -----------------------------------------------------------------------------------------//
		private IntPtr			m_hSsv;
		private	int				m_STX, m_ETX, m_DLE;
		private	int				m_PktTimeout;
		private ESsvRxTextCode	m_RxTextCode =	ESsvRxTextCode.SJIS;
		private ESsvTxTextCode	m_TxTextCode =	ESsvTxTextCode.SJIS;
		private	IntPtr			m_SvLParam	 =	(IntPtr)0;

		//----- RXgN^iPj ---------------------------------------------------------------------------//
		public CAjrSockServer()
		{
			InitializeComponent();

			//	
			InitOnConstruct();
		}

		//----- RXgN^iQj ---------------------------------------------------------------------------//
		public CAjrSockServer(IContainer container)
		{
			container.Add(this);

			InitializeComponent();

			//	
			InitOnConstruct();

		}
		//----- R[h -----------------------------------------------------------------------------------//
		unsafe private void InitOnConstruct()
		{
			//	Cu
			CAjrInit.Init();

			//	ϐ
			m_hSsv 			= (IntPtr)0;
			m_STX			= 0x02;
			m_ETX			= 0x03;
			m_DLE			= 0x10;
			m_PktTimeout	= 3000;

			//	 T[o
			m_SsvCbkRxText		= new SsvCbkRxText		(cbSsvRxText		);
			m_SsvCbkRxEsc		= new SsvCbkRxEsc		(cbSsvRxEsc			);
			m_SsvCbkRxCtrl		= new SsvCbkRxCtrl		(cbSsvRxCtrl		);
			m_SsvCbkRxPacket	= new SsvCbkRxPacket	(cbSsvRxPacket		);
			m_SsvCbkRxNoPkt		= new SsvCbkRxNoPkt		(cbSsvRxNoPkt		);
			m_SsvCbkTxEmpty		= new SsvCbkTxEmpty		(cbSsvTxEmpty		);
			m_SsvCbkRxChunkTxt	= new SsvCbkRxChunkTxt	(cbSsvRxChunkTxt	);
			m_SsvCbkRxChunkBin	= new SsvCbkRxChunkBin	(cbSsvRxChunkBin	);
			m_SsvCbkRxInvChunk	= new SsvCbkRxInvChunk	(cbSsvRxInvChunk	);
			m_SsvCbkConnect		= new SsvCbkConnect		(cbSsvConnect		);
			m_SsvCbkDisconnect	= new SsvCbkDisconnect	(cbSsvDisconnect	);
			m_SsvCbkStart		= new SsvCbkStart		(cbSsvStart			);
			m_SsvCbkStop		= new SsvCbkStop		(cbSsvStop			);
			m_SsvCbkRecvError	= new SsvCbkRecvError	(cbSsvRecvError		);
			m_SsvCbkSendError	= new SsvCbkSendError	(cbSsvSendError		);
			m_SsvCbkGeneralError= new SsvCbkGeneralError(cbSsvGeneralError	);
			m_SsvCbkEnumClients	= new SsvCbkEnumClients	(cbSsvEnumClients	);

			m_hSsv = CAjrApi.AjcSsvCreateForMSIL(m_SsvCbkRxText		,
												 m_SsvCbkRxEsc		,
												 m_SsvCbkRxCtrl		,
												 m_SsvCbkRxPacket	,
												 m_SsvCbkRxNoPkt	,
												 m_SsvCbkTxEmpty	,
												 m_SsvCbkRxChunkTxt	,
												 m_SsvCbkRxChunkBin	,
												 m_SsvCbkRxInvChunk	,
												 m_SsvCbkConnect	,
												 m_SsvCbkDisconnect	,
												 m_SsvCbkRecvError	,
												 m_SsvCbkSendError	,
												 m_SsvCbkStart		,
												 m_SsvCbkStop		,
												 m_SsvCbkGeneralError,
												 m_SsvCbkEnumClients);

			//	ICxgݒ
			this.Disposed += OnUnloadMyControl;
		}
		//----- I ---------------------------------------------------------------------------------------//
		private void OnUnloadMyControl(object sender, EventArgs e)
		{
			if (m_hSsv != (IntPtr)0) {
				CAjrApi.AjcSsvDeleteForMSIL(m_hSsv);
			}
		}
		//------------------------------------------------------------------------------------------------------//
		//	vpeB																							//
		//------------------------------------------------------------------------------------------------------//
		//----- s[h -------------------------------------------------------------------------------------//
		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Event reception mode" +
										   "(This property can be set only at designing)")]
		public ESsvMode _SsvMode {
			set {m_SsvMode = value;}
			get {return m_SsvMode;}
		}
		//----- T[oIvV -------------------------------------------------------------------------------//
		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Power control options")]
		public ESsvServOpt SsvOption {
			set {CAjrApi.AjcSsvSetOpt(m_hSsv, value);}
			get {return CAjrApi.AjcSsvGetOpt(m_hSsv);}
		}
		//----- `Nf[^̈ ---------------------------------------------------------------------------//
		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Kind of reception chunk data" )]
		public ESsvChunkMode ChunkMode {
			set {m_ChunkMode = value; CAjrApi.AjcSsvSetChunkMode(m_hSsv, value);}
			get {return m_ChunkMode;}
		}
		//----- pPbg擪R[h(STX) ------------------------------------------------------------------------//
		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Packet starting code value (STX)")]
		public int STX {
			set {m_STX = (int)value;  CAjrApi.AjcSsvSetPktCtrlCode(m_hSsv, m_STX, 0, 0);}
			get {return (int)m_STX; }
		}
		//----- pPbgR[h(ETX) ------------------------------------------------------------------------//
		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Packer ending code value (ETX)")]
		public int ETX {
			set {m_ETX = (int)value;  CAjrApi.AjcSsvSetPktCtrlCode(m_hSsv, 0, m_ETX, 0);}
			get {return (int)m_ETX; }
		}
		//----- pPbgߐoCgR[h(DLE) --------------------------------------------------------------//
		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Packet transparent control code (DLE)")]
		public int DLE {
			set {m_DLE = (int)value;  CAjrApi.AjcSsvSetPktCtrlCode(m_hSsv, 0, 0, m_DLE);}
			get {return (int)m_DLE; }
		}
		//----- pPbgM^CAEg^C} -----------------------------------------------------------------//
		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Packet reception timeout [ms]")]
		public int PktTimeout {
			set {m_PktTimeout = (int)value;  CAjrApi.AjcSsvSetPktTimeout(m_hSsv, m_PktTimeout);}
			get {return (int)m_PktTimeout; }
		}
		//----- MeLXg̕R[h -----------------------------------------------------------------------//
		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Received text code")]
		public ESsvRxTextCode RxTextCode {
			set {m_RxTextCode = value;	CAjrApi.AjcSsvSetRxTextCode(m_hSsv, m_RxTextCode);
										CAjrApi.AjcSsvSetTxTextCode(m_hSsv, m_TxTextCode);}
			get {return m_RxTextCode; }
		}
		//----- MeLXg̕R[h -----------------------------------------------------------------------//
		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Sending Text code")]
		public ESsvTxTextCode TxTextCode {
			set {m_TxTextCode = value;	CAjrApi.AjcSsvSetRxTextCode(m_hSsv, m_RxTextCode);
										CAjrApi.AjcSsvSetTxTextCode(m_hSsv, m_TxTextCode);}
			get {return m_TxTextCode; }
		}
		//----- ڑNCAg -----------------------------------------------------------------------------//
		[System.ComponentModel.Category("AjrSockServer")]
		[System.ComponentModel.Description("Number of clients")]
		public int Clients {
			get {	int rc;
				 rc = CAjrApi.AjcSsvGetClientCount(m_hSsv);
				 return rc;
			}
		}
		//------------------------------------------------------------------------------------------------------//
		//	\bhiNCAgj																		//
		//------------------------------------------------------------------------------------------------------//
		//----- PoCgM -----------------------------------------------------------------------------------//
		public void SendByte(IntPtr hClient, byte ByteCode)
		{
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				CAjrApi.AjcSsvSendChar(hClient, ByteCode);
			}
			else {throw new ArgumentException("Invalid client handle.");}
		}
		//----- PM -------------------------------------------------------------------------------------//
		public void SendChar(IntPtr hClient, char Character)
		{
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				CAjrApi.AjcSsvSendChar(hClient, (ushort)Character);
			}
			else {throw new ArgumentException("Invalid client handle.");}
		}
		//----- eLXgM -----------------------------------------------------------------------------------//
		public void SendText(IntPtr hClient, string text)
		{
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				CAjrApi.AjcSsvSendText(hClient, text, (int)text.Length);
			}
			else {throw new ArgumentException("Invalid client handle.");}
		}
		//----- oCiM -----------------------------------------------------------------------------------//
		public void SendBinary(IntPtr hClient, Byte[] bin)
		{
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				CAjrApi.AjcSsvSendBinData(hClient, bin);
			}
			else {throw new ArgumentException("Invalid client handle.");}
		}
		//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
		unsafe public void SendBinary(IntPtr hClient, void *p, int len)
		{
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				CAjrApi.AjcSsvSendBinData(hClient, p, len);
			}
			else {throw new ArgumentException("Invalid client handle.");}
		}
		//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
		public void SendBinary(IntPtr hClient, IntPtr p, int len)
		{
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				CAjrApi.AjcSsvSendBinData(hClient, p, len);
			}
			else {throw new ArgumentException("Invalid client handle.");}
		}
		//----- pPbgM -----------------------------------------------------------------------------------//
		public int SendPacket(IntPtr hClient, Byte[] bin)
		{
			int		rc = 0;
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				if (bin != null) {
					rc = CAjrApi.AjcSsvSendPacket(hClient, bin);
				}
				else {
					rc = CAjrApi.AjcSsvSendPacket(hClient, (IntPtr)0, 0);
				}
			}
			else {throw new ArgumentException("Invalid client handle.");}
			return rc;
		}
		//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
		unsafe public int SendPacket(IntPtr hClient, void* p, int len)
		{
			int		rc = 0;
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				rc = CAjrApi.AjcSsvSendPacket(hClient, p, len);
			}
			else {throw new ArgumentException("Invalid client handle.");}
			return rc;
		}
		//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
		public int SendPacket(IntPtr hClient, IntPtr p, int len)
		{
			int		rc = 0;
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				rc = CAjrApi.AjcSsvSendPacket(hClient, p, len);
			}
			else {throw new ArgumentException("Invalid client handle.");}
			return rc;
		}
		//----- Mσf[^j -------------------------------------------------------------------------------//
		public void PurgeRx(IntPtr hClient)
		{
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				CAjrApi.AjcSsvPurgeRecvData(hClient);
			}
			else {throw new ArgumentException("Invalid client handle.");}
		}
		//----- M҂f[^j -----------------------------------------------------------------------------//
		public void PurgeTx(IntPtr hClient)
		{
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				CAjrApi.AjcSsvPurgeSendData(hClient);
			}
			else {throw new ArgumentException("Invalid client handle.");}
		}
		//----- Mf[^j -------------------------------------------------------------------------------//
		public void Purge(IntPtr hClient)
		{
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				CAjrApi.AjcSsvPurgeAllData(hClient);
			}
			else {throw new ArgumentException("Invalid client handle.");}
		}
		//----- NCAgɃf[^֘At -----------------------------------------------------------------//
		public void SetClientData(IntPtr hClient, object obj)
		{
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				if (m_mapClientData.ContainsKey(hClient)) {
					m_mapClientData[hClient] = obj;
				}
				else {
					m_mapClientData.Add(hClient, obj);
				}
			}
			else {throw new ArgumentException("Invalid client handle.");}

		}
		//----- NCAgɊ֘Atꂽf[^擾 ---------------------------------------------------------//
		public void GetClientData(IntPtr hClient, out object obj)
		{
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				if (m_mapClientData.ContainsKey(hClient)) {
					obj = m_mapClientData[hClient];
				}
				else {throw new ArgumentException("Client data is not registered.");}
			}
			else {throw new ArgumentException("Invalid client handle.");}
		}
		//----- NCAgڑԍ擾 -------------------------------------------------------------------//
		public int GetSeqNo(IntPtr hClient)
		{
			int		rc = 0;
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				rc = CAjrApi.AjcSsvGetSeqNo(hClient);
			}
			else {throw new ArgumentException("Invalid client handle.");}
			return rc;
		}
		//----- NCAg̃CfNXl擾 -----------------------------------------------------------------//
		public int GetIndex(IntPtr hClient)
		{
			int		rc = 0;
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				rc = CAjrApi.AjcSsvGetIndex(hClient);
			}
			else {throw new ArgumentException("Invalid client handle.");}
			return rc;
		}
		//----- NCAĝhoAhX擾 -----------------------------------------------------------//
		public string GetIpAddrStr(IntPtr hClient)
		{
			string	rc = null;
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				rc = CAjrApi.AjcSsvGetIpAddrStr(hClient);
			}
			else {throw new ArgumentException("Invalid client handle.");}
			return rc;
		}
		//----- NCAgXbh̃Xbhhc擾 ---------------------------------------------------------//
		public uint GetThreadId(IntPtr hClient)
		{
			uint	rc = 0;
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				rc = CAjrApi.AjcSsvGetThreadId(hClient);
			}
			else {throw new ArgumentException("Invalid client handle.");}
			return rc;
		}
		//----- NCAgؒf -------------------------------------------------------------------------------//
		public void Disconnect(IntPtr hClient)
		{
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				CAjrApi.AjcSsvDisconnect(hClient);
			}
			else {throw new ArgumentException("Invalid client handle.");}
		}
		//----- ۂ̎MeLXg̕R[hiAUTO͎ۂ̎MR[hj ---------------------------------//
		public ESsvRxTextCode GetActualRxTextCode(IntPtr hClient)
		{
			ESsvRxTextCode	rc = 0;
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				rc = CAjrApi.AjcSsvGetActualRxTextCode(hClient);
			}
			else {throw new ArgumentException("Invalid client handle.");}
			return rc;
		}
		//----- ۂ̑MeLXg̕R[hiAUTO͎MR[hj ---------------------------------------//
		public ESsvTxTextCode GetActualTxTextCode(IntPtr hClient)
		{
			ESsvTxTextCode	rc = 0;
			if (CAjrApi.AjcSsvIsClientValid(m_hSsv, hClient)) {
				rc = CAjrApi.AjcSsvGetActualTxTextCode(hClient);
			}
			else {throw new ArgumentException("Invalid client handle.");}
			return rc;
		}
		//------------------------------------------------------------------------------------------------------//
		//	\bhȋj																					//
		//------------------------------------------------------------------------------------------------------//
		//----- T[oJn -------------------------------------------------------------------------------------//
		public void Start(uint PortNo)
		{
			Start(PortNo, ESsvFamily.INET, 10);
		}
		//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
		public void Start(uint PortNo, int MaxClients)
		{
			Start(PortNo, ESsvFamily.INET, MaxClients);
		}
		//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
		public void Start(uint PortNo, ESsvFamily AddressFamily, int MaxClients)
		{
			//	CxgݒiOnXXXXX Cxg)
			if (m_SsvMode == ESsvMode.NotificationByEvent) {
				SetEvtMaskByEvent();
			}
			//	T[oJn
			CAjrApi.AjcSsvStartForMSIL(m_hSsv, PortNo.ToString(), AddressFamily, MaxClients, m_SsvMode == ESsvMode.NotificationByEvent ? true : false);
		}
		//----- T[o~ -------------------------------------------------------------------------------------//
		public void Stop()
		{
			CAjrApi.AjcSsvStop(m_hSsv, 100000);
		}
		//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
		public void Stop(int msTimeout)
		{
			CAjrApi.AjcSsvStop(m_hSsv, msTimeout);
		}
		//----- NCAg -------------------------------------------------------------------------------//
		public	int EnumClients(IntPtr cbp)
		{
			return CAjrApi.AjcSsvEnumClientsForMSIL(m_hSsv, cbp);
		}
		//----- rru -------------------------------------------------------------------------------------//
		public void Delete() {
			if (m_hSsv != (IntPtr)0) {
				CAjrApi.AjcSsvDeleteForMSIL(m_hSsv);
				m_hSsv = (IntPtr)0;
			}
		}
		//----- Cxg}XNݒ -----------------------------------------------------------------------------//
		public void SetEvtMask(ESsvEvt evt)
		{
			if (m_SsvMode == ESsvMode.WaitingForEvent) {	//	R\[AvH
				CAjrApi.AjcSsvSetEvtMask(m_hSsv, (int)evt);
			}
		}
		//----- Cxg}XN擾 -----------------------------------------------------------------------------//
		public ESsvEvt GetEvtMask()
		{
			ESsvEvt	rc = 0;
			rc = (ESsvEvt)CAjrApi.AjcSsvGetEvtMask(m_hSsv);
			return rc;
		}
		//----- Cxg҂ -----------------------------------------------------------------------------------//
		public ESsvEvt WaitEvent(out Object EvtData, int msWaitTime, out IntPtr hClient)
		{
			IntPtr	wParam	= (IntPtr)0;
			IntPtr	lParam	= (IntPtr)0;
			IntPtr	pDat;
			int		len, param;
			ESsvEvt	evt 	= ESsvEvt.EV_NOEVENT;
			IntPtr	pC		= (IntPtr)0;

			//	 o͏NA[
			EvtData = null; 			//	Cxgf[^
			hClient = (IntPtr)0;		//	NCAgnh

			if (m_SvLParam == (IntPtr)0) {
				if (CAjrApi.AjcSsvWaitEvent(m_hSsv, ref wParam, ref lParam, msWaitTime)) {
					//	lParam ޔ
					m_SvLParam = lParam;
					//	Cxgf[^擾
					CAjrApi.AjcSsvGetEventData(m_hSsv, lParam, out pDat, out len, out param);
					//	NCAgEnh擾
					if (CAjrApi.AjcSsvGetClient(m_hSsv, lParam, ref pC)) {
						hClient = pC;
					}
					//	CxgR[hݒ
					evt = (ESsvEvt)wParam;

					//----- eLXgMʒm ---------------------------------------------------------------------//
					if		((evt & ESsvEvt.EV_RXTEXT) != 0) {
						StringBuilder s = new StringBuilder(len + 1);
						CAjrApi.AjcWStrCopy(s, pDat);
						EvtData 	= s.ToString();
						hClient 	= pC;
					}
					//----- drbR[hMʒm -----------------------------------------------------------------//
					else if ((evt & ESsvEvt.EV_RXESC) != 0) {
						StringBuilder s = new StringBuilder(len + 1);
						CAjrApi.AjcWStrCopy(s, pDat);
						EvtData 	= s.ToString();
						hClient 	= pC;
					}
					//----- R[hMʒm -------------------------------------------------------------------//
					else if ((evt & ESsvEvt.EV_RXCTRL) != 0) {
						EvtData 	= CAjrApi.AjcMemPeekUW(pDat);
						hClient 	= pC;
					}
					//----- pPbgf[^Mʒm ---------------------------------------------------------------//
					else if ((evt & ESsvEvt.EV_RXPKT) != 0) {
						Byte[] BArr = new Byte[len];
						CAjrApi.AjcMemCopy(ref BArr[0], pDat, len);
						EvtData 	= BArr;
						hClient 	= pC;
					}
					//----- pPbgOf[^Mʒm -------------------------------------------------------------//
					else if ((evt & ESsvEvt.EV_RXNOPKT) != 0) {
						StringBuilder s = new StringBuilder(len + 1);
						CAjrApi.AjcWStrCopy(s, pDat);
						EvtData 	= s.ToString();
						hClient 	= pC;
					}
					//----- `Nf[^Mʒm ---------------------------------------------------------------//
   	   	   	   		else if ((evt & ESsvEvt.EV_RXCHUNK) != 0) {
						if (param == 0) {
							Byte[] BArr = new Byte[len];
							CAjrApi.AjcMemCopy(ref BArr[0], pDat, len);
							EvtData 	= BArr;
						}
						else {
							StringBuilder s = new StringBuilder(len + 1);
							CAjrApi.AjcWStrCopy(s, pDat);
							EvtData = s.ToString();
						}
						hClient 	= pC;
					}
					//----- s`NeLXgMʒm ---------------------------------------------------------//
					else if ((evt & ESsvEvt.EV_INVCHUNK) != 0) {
						Byte[] BArr = new Byte[len];
						CAjrApi.AjcMemCopy(ref BArr[0], pDat, len);
						EvtData 	= BArr;
						hClient 	= pC;
					}
					//----- Mʒm -------------------------------------------------------------------------//
					else if ((evt & ESsvEvt.EV_TXEMPTY) != 0) {
						EvtData 	= null;
						hClient 	= pC;
					}
					//----- ڑʒm -----------------------------------------------------------------------------//
					else if ((evt & ESsvEvt.EV_CONNECT) != 0) {
						EvtData 	= null;
						hClient 	= pC;
					}
					//----- ؒfʒm -----------------------------------------------------------------------------//
					else if ((evt & ESsvEvt.EV_DISCONNECT) != 0) {
						EvtData 	= null;
						if (m_mapClientData.ContainsKey(hClient)) {
							EvtData = m_mapClientData[hClient];
							m_mapClientData.Remove(hClient);
						}
						hClient 	= pC;
					}
					//----- T[oJnʒm -----------------------------------------------------------------------//
					else if ((evt & ESsvEvt.EV_START) != 0) {
						EvtData 	= null;
						m_mapClientData.Clear();
					}
					//----- T[o~ʒm -----------------------------------------------------------------------//
					else if ((evt & ESsvEvt.EV_STOP) != 0) {
						EvtData 	= null;
						m_mapClientData.Clear();
					}
					//----- ̑ -------------------------------------------------------------------------------//
					else  {
						EvtData 	= param;
					}
				}
				else {	//	^CAEg
					m_SvLParam = (IntPtr)0;
				}
			}
			else {		//	m_SvLParam != 0
				{throw new InvalidOperationException("EndEvent() is not executed after WaitEvent().");}
			}
			return evt;
		}
		//----- Cxg̏I ---------------------------------------------------------------------------------//
		public void EndEvent()
		{
			if (m_SvLParam != (IntPtr)0) {
				CAjrApi.AjcSsvRelEventData(m_hSsv, m_SvLParam);
				m_SvLParam = (IntPtr)0;
			}
		}
		//--------------------------------------------------------------------------------------------------//
		//	Tu֐																					//
		//--------------------------------------------------------------------------------------------------//
		//----- SetEvtMaskByEvent		   -----------------------------------------------------------------//
		private bool SetEvtMaskByEvent()
		{
			bool	rc;
			ESsvEvt	evt = 0;

			if (OnRxText		 != null) evt |= ESsvEvt.EV_RXTEXT;			//	eLXgf[^ʒm
			if (OnRxEsc			 != null) evt |= ESsvEvt.EV_RXESC;			//	drbR[hf[^ʒm
			if (OnRxCtrl		 != null) evt |= ESsvEvt.EV_RXCTRL;			//	R[hʒm
			if (OnRxPacket		 != null) evt |= ESsvEvt.EV_RXPKT;			//	pPbgf[^ʒm
			if (OnRxNoPkt 		 != null) evt |= ESsvEvt.EV_RXNOPKT;		//	pPbgOeLXgf[^
			if (OnRxChunkTxt	 != null) evt |= ESsvEvt.EV_RXCHUNK;		//	pPbgf[^ʒm
			if (OnRxChunkBin	 != null) evt |= ESsvEvt.EV_RXCHUNK;		//	pPbgf[^ʒm
			if (OnRxInvChunk	 != null) evt |= ESsvEvt.EV_INVCHUNK;		//	`Nf[^Mʒm
			if (OnTxEmpty 		 != null) evt |= ESsvEvt.EV_TXEMPTY;		//	Mʒm
			if (OnConnect 		 != null) evt |= ESsvEvt.EV_CONNECT;		//	s`NeLXgMʒm
			if (OnDisconnect	 != null) evt |= ESsvEvt.EV_DISCONNECT;		//	ؒfʒm
			if (OnStart			 != null) evt |= ESsvEvt.EV_START;			//	T[oJnʒm
			if (OnStop			 != null) evt |= ESsvEvt.EV_STOP;			//	T[o~ʒm
			if (OnRecvError		 != null) evt |= ESsvEvt.EV_RXERR;			//	MG[ʒm
			if (OnSendError		 != null) evt |= ESsvEvt.EV_TXERR;			//	MG[ʒm
			if (OnGeneralError	 != null) evt |= ESsvEvt.EV_ERR;			//	G[ʒm

			rc = CAjrApi.AjcSsvSetEvtMask(m_hSsv, (int)evt);
			return rc;
		}
		//------------------------------------------------------------------------------------------------------//
		//	R[obN																						//
		//------------------------------------------------------------------------------------------------------//
		//----- R[obN֐ieLXgMʒmj------------------------------------------------------------//
		private void cbSsvRxText(IntPtr hClient, string text)
		{
			if (OnRxText != null) {
				SsvArgRxText eva = new SsvArgRxText();
				eva.hClient    = hClient;
				eva.text	   = text;
				OnRxText(this, eva);
			}
		}
		//----- R[obN֐idrbMʒmj--------------------------------------------------------------//
		private void cbSsvRxEsc(IntPtr hClient, string esc)
		{
			if (OnRxEsc != null) {
				SsvArgRxEsc eva = new SsvArgRxEsc();
				eva.hClient    = hClient;
				eva.esc 	   = esc;
				OnRxEsc(this, eva);
			}
		}
		//----- R[obN֐iR[hMʒmj----------------------------------------------------------//
		private void cbSsvRxCtrl(IntPtr hClient, char ctrl)
		{
			if (OnRxCtrl != null) {
				SsvArgRxCtrl eva = new SsvArgRxCtrl();
				eva.hClient    = hClient;
				eva.ctrl	   = ctrl;
				OnRxCtrl(this, eva);
			}
		}
		//----- R[obN֐ipPbgMʒmj------------------------------------------------------------//
		unsafe private void cbSsvRxPacket(IntPtr hClient, void *p, int len)
		{
			if (OnRxPacket != null) {
				SsvArgRxPacket eva = new SsvArgRxPacket();
				if (len != 0) {
					eva.hClient = hClient;
					eva.bin 	= new Byte[len];
					fixed (void *pBArr = &eva.bin[0]) {
						CAjrApi.AjcMemCopy(pBArr, p, len);
					}
				}
				else {
					eva.bin = null;
				}
				OnRxPacket(this, eva);
			}

		}
		//----- R[obN֐ipPbgOeLXgMʒmj--------------------------------------------------//
		private void cbSsvRxNoPkt(IntPtr hClient, string text)
		{
			if (OnRxNoPkt != null) {
				SsvArgRxNoPkt eva = new SsvArgRxNoPkt();
				eva.hClient    = hClient;
				eva.text	   = text;
				OnRxNoPkt(this, eva);
			}
		}
		//----- R[obN֐iMʒmj----------------------------------------------------------------//
		private void cbSsvTxEmpty(IntPtr hClient)
		{
			if (OnTxEmpty != null) {
				SsvArgTxEmpty eva = new SsvArgTxEmpty();
				eva.hClient    = hClient;
				OnTxEmpty(this, eva);
			}
		}
		//----- R[obN֐ieLXg`NMʒmj----------------------------------------------------//
		private void cbSsvRxChunkTxt(IntPtr hClient, string text)
		{
			if (OnRxChunkTxt != null) {
				SsvArgRxChunkTxt eva = new SsvArgRxChunkTxt();
				eva.hClient    = hClient;
				eva.text	   = text;
				OnRxChunkTxt(this, eva);
			}
		}
		//----- R[obN֐ioCi`NMʒmj----------------------------------------------------//
		unsafe private void cbSsvRxChunkBin(IntPtr hClient, void *p, int len)
		{
			if (OnRxChunkBin != null) {
				SsvArgRxChunkBin eva = new SsvArgRxChunkBin();
				eva.hClient    = hClient;
				eva.bin = new Byte[len];
				fixed (void *pBArr = &eva.bin[0]) {
					CAjrApi.AjcMemCopy(pBArr, p, len);
				}
				OnRxChunkBin(this, eva);
			}
		}
		//----- R[obN֐is`NeLXgMʒmj------------------------------------------------//
		unsafe private void cbSsvRxInvChunk(IntPtr hClient, void *p, int len)
		{
			if (OnRxInvChunk != null) {
				SsvArgRxInvChunk eva = new SsvArgRxInvChunk();
				eva.hClient = hClient;
				eva.bin 	= new Byte[len];
				fixed (void *pBArr = &eva.bin[0]) {
					CAjrApi.AjcMemCopy(pBArr, p, len);
				}
				OnRxInvChunk(this, eva);
			}
		}
		//----- R[obN֐iڑʒmj--------------------------------------------------------------------//
		private void cbSsvConnect(IntPtr hClient)
		{
			if (OnConnect != null) {
				SsvArgConnect eva = new SsvArgConnect();
				eva.hClient    = hClient;
				OnConnect(this, eva);
			}
		}
		//----- R[obN֐iؒfʒmj--------------------------------------------------------------------//
		private void cbSsvDisconnect(IntPtr hClient)
		{
			if (OnDisconnect != null) {
				SsvArgDisconnect eva = new SsvArgDisconnect();
				eva.hClient    = hClient;
				OnDisconnect(this, eva);
			}
		}
		//----- R[obN֐iMG[ʒmj--------------------------------------------------------------//
		private void cbSsvRecvError(IntPtr hClient, bool overlapped)
		{
			if (OnRecvError != null) {
				SsvArgRecvError eva = new SsvArgRecvError();
				eva.hClient    = hClient;
				eva.overlapped = overlapped;
				OnRecvError(this, eva);
			}
		}
		//----- R[obN֐iMG[ʒmj--------------------------------------------------------------//
		private void cbSsvSendError(IntPtr hClient, bool overlapped)
		{
			if (OnSendError != null) {
				SsvArgSendError eva = new SsvArgSendError();
				eva.hClient    = hClient;
				eva.overlapped = overlapped;
				OnSendError(this, eva);
			}
		}
		//----- R[obN֐ȋ̃G[ʒmj----------------------------------------------------------//
		private void cbSsvGeneralError(ESsvErrorCode code)
		{
			if (OnGeneralError != null) {
				SsvArgGeneralError eva = new SsvArgGeneralError();
				eva.ErrorCode  = code;
				OnGeneralError(this, eva);
			}
		}
		//----- R[obN֐iT[oJnʒmj--------------------------------------------------------------//
		private void cbSsvStart()
		{
			if (OnDisconnect != null) {
				SsvArgStart eva = new SsvArgStart();
				OnStart(this, eva);
			}
		}
		//----- R[obN֐iT[o~ʒmj--------------------------------------------------------------//
		private void cbSsvStop()
		{
			if (OnStop != null) {
				SsvArgStop eva = new SsvArgStop();
				OnStop(this, eva);
			}
		}
		//----- R[obN֐iNCAg񋓁j------------------------------------------------------------//
		private bool cbSsvEnumClients(IntPtr hClient, IntPtr cbp)
		{
			bool	rc = false;

			if (OnEnumClients != null) {
				SsvArgEnumClients eva = new SsvArgEnumClients();
				eva.hClient  = hClient;
				eva.param	 = cbp;
				rc = OnEnumClients(this, eva);
			}
			return rc;
		}
	}
}
